/*
m_miditune - Tune an audio signal to received pitch

DESCRIPTION
m_miditune tunes the input audio signal to the pitch received in kmelody, no
matter what the original pitch has been. This works best on a voice, but it
should work fine on other monophonic instruments as well.

SYNTAX
aout m_miditune ain, ksmooth, keepform, kmelody, kompress, kbypass, ifftsize

INITIALIZATION
ifftsize - Size of the FFT. A good starting value is 1024. This must be a
	power-of-two (e.g. 256, 512, 1024, 2048,...). This causes latency by
	ifftsize samples!

PERFORMANCE
aout - Tuned audio output signal
ain - Audio input signal
ksmooth - Smooth the pitch changes over this time in seconds. Good values
	are between 0.01-0.05 or even lower
keepform - Wheter or not to preserve the formant of the original signal.
	0 = don't preserve formants
	1 = preserve formants
	2 = preserve formants at high quality (not realtime capable)
kmelody - Control signal holding MIDI note numbers of the pitch you want
	the input signal at
kompress - Ratio of a compressor. 1 means no compression, greater 1 means
	compress the input signal. May be very helpful!
kbypass - Bypass the processing, if this is not zero.

CREDITS
Author: Jeanette C., based on code supplied by Victor Lazzarini. Thank you to
the Csound community for the constant support and helpful spirit!
*/

opcode m_miditune, a, akkkkki
ain, ksmooth, keepform, kmelody, kompress, kbypass, ifftsize xin

	klastfreq init 330 ; last frequency, after processing
	ibasefreq init 220 ; base frequency for note calculation
	ibasemidinote init 57 ; MIDI note number corresponding to ibasefreq

	if kompress == 1 then ; Don't use the compressor, if ratio is 1
		acomp = ain
	else ; Compress the input, sometimes very helpful
		acomp compress ain, ain, 8, 48, 60, kompress, 0.0001, 0.35, 0.01
		acomp = acomp * 3 * kompress
	endif

	; Delay the signal by FFT, so we get a smooth cross between bypass/process
	abuf delayr 0.1
	adel deltapn ifftsize
	delayw ain

	if kbypass == 0 then
		; FFT analyse and pitch track the input signal
		fsrc pvsanal acomp, ifftsize, ifftsize / 4, ifftsize, 1
		fsig pvslock fsrc, 1
		kfreq, kamp pvspitch fsig, 0.05
		if kfreq > 20 then ; Frequencies below 10Hz are undesired/errors
			kgoto Process
		else
			kfreq = klastfreq
		endif
	else ; Output the input signal delayed, so it alligns to the analysed one
		aout = adel
		kgoto Output
	endif

Process:
	if kmelody == 0 then
		krealnote = klastfreq
	else
		krealnote = cpsmidinn(kmelody)
	endif
	klastfreq = krealnote
	kratio = krealnote / kfreq
	kratiofilt portk kratio, ksmooth, ibasefreq
	fpitched pvscale fsig, kratiofilt, keepform
	atuned pvsynth fpitched
	aout balance atuned, adel
	kgoto Output

Output:
	xout aout
endop
