/*
m_pitchcorrect - pitch correct an audio signal

DESCRIPTION
m_pitchcorrect can correct the pitch to the nearest well-tempered note. It
works best on a voice, though other monophonic instruments can be pitch
corrected as well.

SYNTAX
aout m_pitchcorrect ain, ksmooth, ktrans, keepform, kstr4ength, khold, \
		kompress, kbypass, ifftsize

INITIALIZATION
ifftsize - Size of the FFT, a good value to start is 1024. This must be
	a power-of-two (e.g. 256, 512, 1024, 2048,...). This causes latecy of
	ifftsize samples

PERFORMANCE
aout - Pitch corrected audio output signal
ain - Audio input signal
ksmooth - Smoothing time of the effect in seconds. Good starting values are
	between 0.01-0.05 or lower
ktrans - Notes to transpose the original signal by. Positive numbers transpose
	upwards, negative numbers downards. 1 means one semitone up
keepform - Whether or not to preserve the original formants (avoid the chipmunk
	effect).
	0 = don't preserve
	1 = preserve
	2 = preserve with high quality (not realtime capable)
kstrength - Strength of the correction effect, between 0 and 1
khold - Sample and hold the current detected pitch at a rate of khold Hertz.
	100 is a good starting value to smooth notes. Strangely a rate of kr causes
	problems
kompress - Ratio of a compressor. 1 means no compression, great then 1 means
	compress the input signal. May be very helpful!
kbypass - Bypass the processing, if this is not zero

CREDITS
Author: Jeanette C. based on code provided by Victor Lazzarini. Thank you to
the Csound community for the valued support!
*/

opcode m_pitchcorrect, a, akkkkkkki
ain, ksmooth, ktrans, keepform, kstrength, khold, kompress, kbypass, ifftsize xin
	klastfreq init 110 ; last frequency, after processing
	ibasefreq init 220 ; base frequency for note calculation
	ibasemidinote init 57 ; MIDI note number corresponding to ibasefreq

	if kompress == 1 then ; No compression
		acomp = ain
	else ; Compress the signal
		acomp compress ain, ain, 8, 48, 60, kompress, 0.0001, 0.35, 0.001
		acomp = acomp * 3 * kompress
	endif

	; Delay the input signal, so bypass will be smooth
	abuf delayr 0.1
	adel deltapn ifftsize
	delayw acomp

	if kbypass == 0 then
		; FFT analyse the audio signal and run the pitch tracker on it
		fsrc pvsanal acomp, ifftsize, ifftsize / 4, ifftsize, 1
		fsig pvslock fsrc, 1
		kfreq, kamp pvspitch fsig, 0.04
		if kfreq > 10 then ; below 10Hz is not a wanted frequency/an error
			kgoto Process
		else
			kfreq = klastfreq
		endif
	else ; Output the delayed input signal so it alligns to the analysed one
		aout = adel
		kgoto Output
	endif

; Do the real work
Process:
	; Set up the Sample and HOld
	kphs phasor khold
	ktrig diff (1 - kphs)
	ktmpmidinote = round(12 * (log(kfreq / ibasefreq) / log(2)) + ibasemidinote + ktrans)
		; calculate the nearest MIDI note, so we can get a frequency from that
	kmidinote samphold ktmpmidinote, ktrig
	krealnote = cpsmidinn(kmidinote)
	klastfreq = krealnote
	ktmpratio = krealnote / kfreq
	kratio = (ktmpratio * kstrength) + (1 - kstrength) ; the correction factor for the original frequency
	kratiofilt portk kratio, ksmooth, ibasefreq ; smooth it
	fpitched pvscale fsig, kratiofilt, keepform ; shift it
	adone pvsynth fpitched
	aout balance adone, adel
	kgoto Output

Output:
	xout aout
endop
