/*
m_vocoder - A channel vocoder using an FFT based technique

DESCRIPTION
m_vocoder is a channel vocoder implemented using an FFT based technique.

SYNTAX
aout m_vocoder acarrier, amodulator, kbands, kompress, kbypass, ifftsize

INITIALIZATION
ifftsize - Size of the FFT. This must be a power-of-two (e.g. 512, 1024, 2048,
	...). Good values to start with are 1024 and 2048. This causes latency of
	ifftsize samples!

PERFORMANCE
aout - Output of the vocoder
acarrier - The carrier of the vocodeer, that which determines pitch. Typically
	a synthesizer sound (a pad, strings,...)
amodulator - The modulator of the vocoder, that which determines formants. This
	is typically a voice
kbands - The number of vocoding bands. Between 40 and 80 is a good value range.
	It's actually number of cepstra coefficients (see pvsvoc)
kompress - Ratio of a compressor. 1 means no compression, above 1 the modulator
	gets compressed before passing into the vocoder. This can be helpful!
kbypass - Bypass processing, if this is not zero

CREDITS
Author: Jeanette C., thank you to the very supportive Csound community and
to all developers and contributors for such a powerful software!
*/

opcode m_vocoder, a, aakkki
acarrier, amodulator, kbands, kompress, kbypass, ifftsize xin

	if kbypass == 0 then ; Only process the signal if not bypassed
		if kompress == 1 then ; Only use the compressor if ratio is not 1
			acompmod = amodulator
		else ; Compress and apply makeup gain
			acompmod compress amodulator, amodulator, 8, 48, 60, kompress, 0.0001, 0.35, 0.01
			acompmod = acompmod * 2.2 * kompress
		endif

		; FFT analyse the input signals
		fmodulator pvsanal acompmod, ifftsize, ifftsize/4, ifftsize, 1
		fcarrier pvsanal acarrier, ifftsize, ifftsize/4, ifftsize, 1
		fvoc pvsvoc fmodulator, fcarrier, 1, 1, kbands
		aout pvsynth fvoc

		; Delay the original audio voice input by the size of the fft, so
		; it alligns with the processed signal
		abuf delayr 0.1
		adel deltapn ifftsize
		delayw acompmod
		aout balance aout, adel ; adapt the volume of the vocoder output
		kgoto Output
	else ; Output the origina signal, because we're bypassed
		aout = amodulator
		kgoto Output
	endif

Output:
	xout aout
endop
